@@ -24,7 +24,7 @@ from guardian.exceptions import MixedContentTypeError
 from guardian.exceptions import NotUserNorGroup
 from guardian.exceptions import WrongAppError
 from guardian.exceptions import MultipleIdentityAndObjectError
-from guardian.testapp.models import NonIntPKModel
+from guardian.testapp.models import CharPKModel, ChildTestModel, UUIDPKModel
 from guardian.testapp.tests.test_core import ObjectPermissionTestCase
 from guardian.models import Group, Permission
 
@@ -134,6 +134,34 @@ class AssignPermTest(ObjectPermissionTestCase):
             self.assertEqual(len(warns), 1)
             self.assertTrue(isinstance(warns[0].message, DeprecationWarning))
 
+    def test_user_assign_perm_list(self):
+        """
+        Test that one is able to assign permissions for a list of objects
+        to a user
+        """
+        assign_perm("add_contenttype", self.user, self.ctype_list)
+        assign_perm("change_contenttype", self.group, self.ctype_list)
+        assign_perm(self.get_permission("delete_contenttype"), self.user, self.ctype_list)
+        for obj in self.ctype_list:
+            self.assertTrue(self.user.has_perm("add_contenttype", obj))
+            self.assertTrue(self.user.has_perm("change_contenttype", obj))
+            self.assertTrue(self.user.has_perm("delete_contenttype", obj))
+
+    def test_group_assign_perm_list(self):
+        """
+        Test that one is able to assign permissions for a list of
+        objects to a group
+        """
+        assign_perm("add_contenttype", self.group, self.ctype_list)
+        assign_perm("change_contenttype", self.group, self.ctype_list)
+        assign_perm(self.get_permission("delete_contenttype"), self.group, self.ctype_list)
+
+        check = ObjectPermissionChecker(self.group)
+        for obj in self.ctype_list:
+            self.assertTrue(check.has_perm("add_contenttype", obj))
+            self.assertTrue(check.has_perm("change_contenttype", obj))
+            self.assertTrue(check.has_perm("delete_contenttype", obj)) 
+
 
 class MultipleIdentitiesOperationsTest(ObjectPermissionTestCase):
     """
@@ -972,34 +1000,49 @@ class GetObjectsForUser(TestCase):
             set(objects.values_list('id', flat=True)),
             {ctypes[0].id, ctypes[1].id})
 
-    def test_non_integer_primary_key(self):
+    def test_varchar_primary_key(self):
         """
         Verify that the function works when the objects that should be returned
-        have non-integer primary keys.
+        have varchar primary keys.
         """
-        obj_with_char_pk = NonIntPKModel.objects.create(
+        obj_with_char_pk = CharPKModel.objects.create(
             char_pk='testprimarykey')
-        assign_perm('add_nonintpkmodel', self.user, obj_with_char_pk)
+        assign_perm('add_charpkmodel', self.user, obj_with_char_pk)
 
-        objects = get_objects_for_user(self.user, 'testapp.add_nonintpkmodel')
+        objects = get_objects_for_user(self.user, 'testapp.add_charpkmodel')
         self.assertEqual(len(objects), 1)
         self.assertTrue(isinstance(objects, QuerySet))
         self.assertEqual(
             set(objects.values_list('pk', flat=True)),
             {obj_with_char_pk.pk})
 
-    def test_non_integer_primary_key_with_any_perm(self):
+    def test_uuid_primary_key(self):
+        """
+        Verify that the function works when the objects that should be returned
+        have uuid primary keys.
+        """
+        obj_with_uuid_pk = UUIDPKModel.objects.create()
+        assign_perm('add_uuidpkmodel', self.user, obj_with_uuid_pk)
+
+        objects = get_objects_for_user(self.user, 'testapp.add_uuidpkmodel')
+        self.assertEqual(len(objects), 1)
+        self.assertTrue(isinstance(objects, QuerySet))
+        self.assertEqual(
+            set(objects.values_list('pk', flat=True)),
+            {obj_with_uuid_pk.pk})
+
+    def test_varchar_primary_key_with_any_perm(self):
         """
         Verify that the function works with any_perm set to True when the
-        objects that should be returned have non-integer primary keys.
+        objects that should be returned have varchar primary keys.
         """
-        obj_with_char_pk = NonIntPKModel.objects.create(
+        obj_with_char_pk = CharPKModel.objects.create(
             char_pk='testprimarykey')
-        assign_perm('add_nonintpkmodel', self.user, obj_with_char_pk)
+        assign_perm('add_charpkmodel', self.user, obj_with_char_pk)
 
         objects = get_objects_for_user(
             self.user,
-            ['testapp.add_nonintpkmodel', 'testapp.change_nonintpkmodel'],
+            ['testapp.add_charpkmodel', 'testapp.change_charpkmodel'],
             any_perm=True)
         self.assertEqual(len(objects), 1)
         self.assertTrue(isinstance(objects, QuerySet))
@@ -1007,20 +1050,38 @@ class GetObjectsForUser(TestCase):
             set(objects.values_list('pk', flat=True)),
             {obj_with_char_pk.pk})
 
-    def test_non_integer_primary_key_with_group_values(self):
+    def test_uuid_primary_key_with_any_perm(self):
+        """
+        Verify that the function works with any_perm set to True when the
+        objects that should be returned have uuid primary keys.
+        """
+        obj_with_uuid_pk = UUIDPKModel.objects.create()
+        assign_perm('add_uuidpkmodel', self.user, obj_with_uuid_pk)
+
+        objects = get_objects_for_user(
+            self.user,
+            ['testapp.add_uuidpkmodel', 'testapp.change_uuidpkmodel'],
+            any_perm=True)
+        self.assertEqual(len(objects), 1)
+        self.assertTrue(isinstance(objects, QuerySet))
+        self.assertEqual(
+            set(objects.values_list('pk', flat=True)),
+            {obj_with_uuid_pk.pk})
+
+    def test_varchar_primary_key_with_group_values(self):
         """
         Verify that the function works when the objects that should be returned
-        have non-integer primary keys, and those objects are due to the user's
+        have varchar primary keys, and those objects are due to the user's
         groups.
         """
-        obj_with_char_pk = NonIntPKModel.objects.create(
+        obj_with_char_pk = CharPKModel.objects.create(
             char_pk='testprimarykey')
-        assign_perm('add_nonintpkmodel', self.group, obj_with_char_pk)
+        assign_perm('add_charpkmodel', self.group, obj_with_char_pk)
         self.user.groups.add(self.group)
 
         objects = get_objects_for_user(
             self.user,
-            ['testapp.add_nonintpkmodel', 'testapp.change_nonintpkmodel'],
+            ['testapp.add_charpkmodel', 'testapp.change_charpkmodel'],
             any_perm=True)
         self.assertEqual(len(objects), 1)
         self.assertTrue(isinstance(objects, QuerySet))
@@ -1028,6 +1089,37 @@ class GetObjectsForUser(TestCase):
             set(objects.values_list('pk', flat=True)),
             {obj_with_char_pk.pk})
 
+    def test_model_inheritance(self):
+        child_with_perm = ChildTestModel.objects.create(name="child1")
+        assign_perm('testapp.change_childtestmodel', self.user, child_with_perm)
+        child_without_perm = ChildTestModel.objects.create(name="child2")
+
+        children = get_objects_for_user(self.user, 'testapp.change_childtestmodel', ChildTestModel)
+
+        self.assertEqual(1, len(children))
+        self.assertIn(child_with_perm, children)
+        self.assertNotIn(child_without_perm, children)
+
+    def test_uuid_primary_key_with_group_values(self):
+        """
+        Verify that the function works when the objects that should be returned
+        have uuid primary keys, and those objects are due to the user's
+        groups.
+        """
+        obj_with_uuid_pk = UUIDPKModel.objects.create()
+        assign_perm('add_uuidpkmodel', self.group, obj_with_uuid_pk)
+        self.user.groups.add(self.group)
+
+        objects = get_objects_for_user(
+            self.user,
+            ['testapp.add_uuidpkmodel', 'testapp.change_uuidpkmodel'],
+            any_perm=True)
+        self.assertEqual(len(objects), 1)
+        self.assertTrue(isinstance(objects, QuerySet))
+        self.assertEqual(
+            set(objects.values_list('pk', flat=True)),
+            {obj_with_uuid_pk.pk})
+
     def test_exception_different_ctypes(self):
         self.assertRaises(MixedContentTypeError, get_objects_for_user,
                           self.user, ['auth.change_permission', 'auth.change_group'])
